SimpleView subclass:#UpDownButton
	instanceVariableNames:'orientation upButton downButton'
	classVariableNames:'DefaultDownForm DefaultUpForm DefaultLeftForm DefaultRightForm'
	poolDictionaries:''
	category:'Views-Interactors'
!

!UpDownButton class methodsFor:'documentation'!

documentation
"
    an up/down button - simply two buttons in one view.

    [author:]
        Claus Gittinger

    [see also:]
        ArrowButton
        ComboUpDownView
"
!

examples
"
                                                                [exBegin]
     |top ud|

     top := StandardSystemView new.
     top extent:(300 @ 200).

     ud := UpDownButton in:top.
     ud origin:(10 @ 10).

     ud upAction:[Transcript showCR:'up'].
     ud downAction:[Transcript showCR:'down'].
     top open.
                                                                [exEnd]

                                                                [exBegin]
     |top ud|

     top := StandardSystemView new.
     top extent:(300 @ 200).

     ud := UpDownButton in:top.
     ud orientation:#horizontal.
     ud origin:(10 @ 10).

     ud upAction:[Transcript showCR:'up'].
     ud downAction:[Transcript showCR:'down'].
     top open.
                                                                [exEnd]
"

! !

!UpDownButton class methodsFor:'defaults'!

downButtonForm
    "return the down-buttons image"

    <resource: #style (#upDownButton.downForm #upDownButton.downFormFile)>

    |fileName form|

    DefaultDownForm notNil ifTrue:[
        ^ DefaultDownForm
    ].

    form := StyleSheet at:'upDownButton.downForm' default:nil.
    form isNil ifTrue:[
        fileName := StyleSheet at:'upDownButton.downFormFile' default:'ComboDnB_win.xbm'.
        form := Image fromFile:fileName.
    ].
    form isNil ifTrue:[
        form  := Form width:10 height:6 
                      fromArray:#[
                                  2r00000000 2r00000000
                                  2r01111111 2r10000000
                                  2r00111111 2r00000000
                                  2r00011110 2r00000000
                                  2r00001100 2r00000000
                                  2r00000000 2r00000000
                                 ]
                      on:Display.
    ].
    form notNil ifTrue:[
        DefaultDownForm := form.
        ^ form onDevice:Display
    ].
    ^ nil

    "Created: / 1.4.1997 / 11:53:39 / cg"
    "Modified: / 25.10.1997 / 14:24:20 / cg"
    "Modified: / 8.1.1998 / 16:33:54 / stefan"
!

leftButtonForm
    "return the left-buttons image"

    <resource: #style (#upDownButton.leftForm #upDownButton.leftFormFile)>

    |fileName form|

    DefaultLeftForm notNil ifTrue:[
        ^ DefaultLeftForm
    ].

    form := StyleSheet at:'upDownButton.leftForm' default:nil.
    form isNil ifTrue:[
        fileName := StyleSheet at:'upDownButton.leftFormFile' default:'ComboLtB_win.xbm'.
        form := Image fromFile:fileName.
    ].
    form isNil ifTrue:[
        form  := Form width:6 height:10 
                      fromArray:#[
                                  2r00000000
                                  2r00001000
                                  2r00011000
                                  2r00111000
                                  2r01111000
                                  2r01111000
                                  2r00111000
                                  2r00011000
                                  2r00001000
                                  2r00000000
                                 ]
                      on:Display.
    ].
    form notNil ifTrue:[
        DefaultLeftForm := form.
        ^ form onDevice:Display
    ].
    ^ nil

    "Created: / 1.4.1997 / 12:39:18 / cg"
    "Modified: / 25.10.1997 / 14:21:45 / cg"
    "Modified: / 8.1.1998 / 16:34:05 / stefan"
!

rightButtonForm
    "return the left-buttons image"

    <resource: #style (#upDownButton.rightForm #upDownButton.rightFormFile)>

    |fileName form|

    DefaultRightForm notNil ifTrue:[
        ^ DefaultRightForm
    ].

    form := StyleSheet at:'upDownButton.rightForm' default:nil.
    form isNil ifTrue:[
        fileName := StyleSheet at:'upDownButton.rightFormFile' default:'ComboRtB_win.xbm'.
        form := Image fromFile:fileName.
    ].
    form isNil ifTrue:[
        form  := Form width:6 height:10 
                      fromArray:#[
                                  2r00000000
                                  2r01000000
                                  2r01100000
                                  2r01110000
                                  2r01111000
                                  2r01111000
                                  2r01110000
                                  2r01100000
                                  2r01000000
                                  2r00000000
                                 ]
                      on:Display.
    ].
    form notNil ifTrue:[
        DefaultRightForm := form.
        ^ form onDevice:Display
    ].
    ^ nil

    "Created: / 1.4.1997 / 12:39:34 / cg"
    "Modified: / 25.10.1997 / 14:23:04 / cg"
    "Modified: / 8.1.1998 / 16:34:15 / stefan"
!

upButtonForm
    "return the up-buttons image"

    <resource: #style (#upDownButton.upForm #upDownButton.upFormFile)>

    |fileName form|

    DefaultUpForm notNil ifTrue:[
        ^ DefaultUpForm
    ].

    form := StyleSheet at:'upDownButton.upForm' default:nil.
    form isNil ifTrue:[
        fileName := StyleSheet at:'upDownButton.upFormFile' default:'ComboUpB_win.xbm'.
        form := Image fromFile:fileName.
    ].
    form isNil ifTrue:[
        form  := Form width:10 height:6 
                      fromArray:#[
                                  2r00000000 2r00000000
                                  2r00001100 2r00000000
                                  2r00011110 2r00000000
                                  2r00111111 2r00000000
                                  2r01111111 2r10000000
                                  2r00000000 2r00000000
                                 ]
                      on:Display.
    ].
    form notNil ifTrue:[
        DefaultUpForm := form.
        ^ form onDevice:Display
    ].
    ^ nil

    "Created: / 1.4.1997 / 11:54:13 / cg"
    "Modified: / 25.10.1997 / 14:23:49 / cg"
    "Modified: / 8.1.1998 / 16:34:36 / stefan"
!

updateStyleCache
    DefaultDownForm := DefaultUpForm := DefaultLeftForm := DefaultRightForm := nil.

    "Created: / 25.10.1997 / 14:11:48 / cg"
! !

!UpDownButton methodsFor:'accessing-behavior'!

downAction:aBlock
    downButton action:aBlock

    "Created: 1.4.1997 / 11:56:33 / cg"
!

upAction:aBlock
    upButton action:aBlock

    "Created: 1.4.1997 / 11:56:28 / cg"
! !

!UpDownButton methodsFor:'accessing-components'!

downButton
    ^ downButton

    "Created: 1.4.1997 / 11:56:05 / cg"
!

upButton
    ^ upButton

    "Created: 1.4.1997 / 11:55:52 / cg"
    "Modified: 1.4.1997 / 11:56:02 / cg"
! !

!UpDownButton methodsFor:'accessing-look'!

orientation
    ^ orientation

    "Created: 1.4.1997 / 12:42:01 / cg"
!

orientation:aSymbol
    orientation := aSymbol.
    self initializeButtonDimensions

    "Created: 1.4.1997 / 12:42:17 / cg"
    "Modified: 1.4.1997 / 12:42:29 / cg"
! !

!UpDownButton methodsFor:'initialization'!

initialize
    orientation := #vertical.

    super initialize.
    self initializeButtons

    "Created: 1.4.1997 / 11:58:07 / cg"
    "Modified: 1.4.1997 / 12:47:58 / cg"
!

initializeButtonDimensions
    orientation == #vertical ifTrue:[
        upButton label:(self class upButtonForm).
        downButton label:(self class downButtonForm).

        upButton origin:0@0.
        downButton origin:(0 @ upButton preferredExtent y).

        self preferredExtent:(upButton preferredExtent x
                              @
                              (upButton preferredExtent y 
                              +
                              downButton preferredExtent y)).
    ] ifFalse:[
        upButton label:(self class rightButtonForm).
        downButton label:(self class leftButtonForm).

        downButton origin:0@0.
        upButton origin:(downButton preferredExtent x @ 0).

        self preferredExtent:((upButton preferredExtent x
                              +
                              downButton preferredExtent x)
                              @
                              upButton preferredExtent y).
    ].

    "Modified: 1.4.1997 / 12:47:38 / cg"
!

initializeButtons
    upButton := ArrowButton upIn:self.
    upButton action:[self countUp].
    upButton autoRepeat:true.

    downButton := ArrowButton downIn:self.
    downButton action:[self countDown].
    downButton autoRepeat:true.

    self initializeButtonDimensions

    "Modified: 1.4.1997 / 12:42:58 / cg"
! !

!UpDownButton class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/libwidg2/UpDownButton.st,v 1.4 1998-01-09 23:51:27 stefan Exp $'
! !
